package gs.log.v0

import java.time.Instant
import scala.collection.mutable.{Map => MutMap}

/**
  * Representation of a Log - a package of data that can be rendered and emitted
  * by a [[Logger]] and underlying engine. This type provides a fluent interface
  * for constructing and logs.
  *
  * @param data Key/value mapping of data that is safe to log.
  * @param tags List of tags (see: [[LogTags]]) applicable to this log.
  * @param message Optional [[LogMessage]] to include in the output.
  * @param exception Optional exception to include in the output.
  */
final class Log(
  private val internalData: MutMap[LogData.Key, LogData.Safe] = MutMap.empty,
  private var internalTags: List[String] = List.empty,
  private var internalMessage: Option[LogMessage] = None,
  private var internalException: Option[Throwable] = None
):
  def data(key: LogData.Key, value: LogData.Safe): Log =
    val _ = this.internalData.put(key, value)
    this

  def data(kvp: (LogData.Key, LogData.Safe)*): Log =
    val _ = kvp.foreach {
      case (k, v) => val _ = this.internalData.put(k, v)
    }
    this

  def tagged(newTags: String*): Log =
    this.internalTags = newTags.toList
    this

  def message(newMessage: LogMessage): Log =
    this.internalMessage = Some(newMessage)
    this

  def exception(ex: Throwable): Log =
    this.internalException = Some(ex)
    this

  def getData: Map[LogData.Key, LogData.Safe] = internalData.toMap

  def getTags: List[String] = internalTags

  def getMessage: Option[LogMessage] = internalMessage

  def getException: Option[Throwable] = internalException

object Log:
  def data(key: LogData.Key, value: LogData.Safe): Log =
    new Log().data(key, value)

  def data(kvp: (LogData.Key, LogData.Safe)*): Log =
    new Log().data(kvp*)

  def tagged(tags: String*): Log =
    new Log().tagged(tags*)

  def message(msg: LogMessage): Log =
    new Log().message(msg)

  def exception(ex: Throwable): Log =
    new Log().exception(ex)

  /** All supplementary information about a [[Log]] as generated/passed by the
    * [[Logger]]. This type should not typically be used directly by users.
    *
    * @param level
    *   The selected [[LogLevel]]
    * @param timestamp
    *   The instant the [[Log]] was _submitted_.
    * @param owner
    *   The name of the [[Logger]] which submitted the log.
    * @param trace Optional [[LogTrace]] for a specific call.
    */
  case class Metadata(
    level: LogLevel,
    timestamp: Instant,
    owner: Logger.Name,
    trace: Option[LogTrace]
  )

end Log
