package gs.log.v0

/** Opaque type (String) for representing messages that should be logged. This
  * type can _only_ be instantiated using the `log` string interpolator, which
  * allows it to be (relatively) "safe" in that users _must_ explicitly declare
  * all data they intend to log as safe for clear text.
  *
  * Example:
  * {{{
  * val data = "example".logClearText()
  * val message: LogMessage = log"this is an example '$data'"
  * }}}
  */
opaque type LogMessage = String

object LogMessage:

  given CanEqual[LogMessage, LogMessage] = CanEqual.derived

  extension (logMessage: LogMessage)
    /** Express this [[LogMessage]] as a string.
      *
      * @return
      *   The string rendition of this [[LogMessage]].
      */
    def str(): String = logMessage

  trait Builder:

    extension (sc: StringContext)

      /** String interpolator for instantiating a [[LogMessage]].
        *
        * @param args
        *   All interpolation variables. Must be [[LogData.ClearText]]
        *   instances.
        * @return
        *   The new [[LogMessage]].
        */
      def log(args: LogData.ClearText*): LogMessage =
        sc.s(args.map(_.value.render())*)

end LogMessage
